@@ -149,6 +149,7 @@ typedef enum {
     QEMU_FIRMWARE_FEATURE_AMD_SEV,
     QEMU_FIRMWARE_FEATURE_AMD_SEV_ES,
     QEMU_FIRMWARE_FEATURE_AMD_SEV_SNP,
+    QEMU_FIRMWARE_FEATURE_INTEL_TDX,
     QEMU_FIRMWARE_FEATURE_ENROLLED_KEYS,
     QEMU_FIRMWARE_FEATURE_REQUIRES_SMM,
     QEMU_FIRMWARE_FEATURE_SECURE_BOOT,
@@ -167,6 +168,7 @@ VIR_ENUM_IMPL(qemuFirmwareFeature,
               "amd-sev",
               "amd-sev-es",
               "amd-sev-snp",
+              "intel-tdx",
               "enrolled-keys",
               "requires-smm",
               "secure-boot",
@@ -1054,17 +1056,24 @@ qemuFirmwareEnsureNVRAM(virDomainDef *def,
     if (loader->nvram && !loader->nvram->format)
         loader->nvram->format = loader->format;
 
-    /* If the source already exists and is fully specified, including
-     * the path, leave it alone */
-    if (loader->nvram && loader->nvram->path)
-        return;
+    if (loader->nvram) {
+        /* Nothing to do if a proper NVRAM backend is already configured */
+        if (!virStorageSourceIsEmpty(loader->nvram))
+            return;
 
-    if (loader->nvram)
+        /* otherwise we want to reset and re-populate the definition */
         virObjectUnref(loader->nvram);
+    }
 
     loader->nvram = virStorageSourceNew();
     loader->nvram->type = VIR_STORAGE_TYPE_FILE;
-    loader->nvram->format = loader->format;
+
+    /* The nvram template format should be always present but as a failsafe,
+     * duplicate the loader format if it is not available. */
+    if (loader->nvramTemplateFormat > VIR_STORAGE_FILE_NONE)
+        loader->nvram->format = loader->nvramTemplateFormat;
+    else
+        loader->nvram->format = loader->format;
 
     if (loader->nvram->format == VIR_STORAGE_FILE_RAW) {
         /* The extension used by raw edk2 builds has historically
@@ -1151,6 +1160,7 @@ qemuFirmwareMatchDomain(const virDomainDef *def,
     bool supportsSEV = false;
     bool supportsSEVES = false;
     bool supportsSEVSNP = false;
+    bool supportsTDX = false;
     bool supportsSecureBoot = false;
     bool hasEnrolledKeys = false;
     int reqSecureBoot;
@@ -1202,6 +1212,10 @@ qemuFirmwareMatchDomain(const virDomainDef *def,
             supportsSEVSNP = true;
             break;
 
+        case QEMU_FIRMWARE_FEATURE_INTEL_TDX:
+            supportsTDX = true;
+            break;
+
         case QEMU_FIRMWARE_FEATURE_REQUIRES_SMM:
             requiresSMM = true;
             break;
@@ -1325,13 +1339,6 @@ qemuFirmwareMatchDomain(const virDomainDef *def,
                           flash->nvram_template.format);
                 return false;
             }
-            if (loader && loader->nvram && loader->nvram->format &&
-                STRNEQ(flash->nvram_template.format, virStorageFileFormatTypeToString(loader->nvram->format))) {
-                VIR_DEBUG("Discarding loader with mismatching nvram template format '%s' != '%s'",
-                          flash->nvram_template.format,
-                          virStorageFileFormatTypeToString(loader->nvram->format));
-                return false;
-            }
         }
     } else if (fw->mapping.device == QEMU_FIRMWARE_DEVICE_MEMORY) {
         if (loader && loader->type &&
@@ -1370,8 +1377,18 @@ qemuFirmwareMatchDomain(const virDomainDef *def,
                 return false;
             }
             break;
+
+        case VIR_DOMAIN_LAUNCH_SECURITY_TDX:
+            if (!supportsTDX) {
+                VIR_DEBUG("Domain requires TDX, firmware '%s' doesn't support it",
+                          path);
+                return false;
+            }
+            break;
+
         case VIR_DOMAIN_LAUNCH_SECURITY_PV:
             break;
+
         case VIR_DOMAIN_LAUNCH_SECURITY_NONE:
         case VIR_DOMAIN_LAUNCH_SECURITY_LAST:
             virReportEnumRangeError(virDomainLaunchSecurity, def->sec->sectype);
@@ -1421,8 +1438,16 @@ qemuFirmwareEnableFeaturesModern(virDomainDef *def,
              * We can't create or reset non-local NVRAM files, so filling
              * in nvramTemplate for those would be misleading */
             VIR_FREE(loader->nvramTemplate);
-            if (!loader->nvram ||
-                (loader->nvram && virStorageSourceIsLocalStorage(loader->nvram))) {
+            loader->nvramTemplateFormat = VIR_STORAGE_FILE_NONE;
+
+            if (!loader->nvram || virStorageSourceIsLocalStorage(loader->nvram)) {
+                /* validation when parsing the JSON files ensures that we get
+                 * only 'raw' and 'qcow2' here. Fall back to sharing format with loader */
+                if (flash->nvram_template.format)
+                    loader->nvramTemplateFormat = virStorageFileFormatTypeFromString(flash->nvram_template.format);
+                else
+                    loader->nvramTemplateFormat = loader->format;
+
                 loader->nvramTemplate = g_strdup(flash->nvram_template.filename);
             }
         }
@@ -1481,6 +1506,7 @@ qemuFirmwareEnableFeaturesModern(virDomainDef *def,
         case QEMU_FIRMWARE_FEATURE_AMD_SEV:
         case QEMU_FIRMWARE_FEATURE_AMD_SEV_ES:
         case QEMU_FIRMWARE_FEATURE_AMD_SEV_SNP:
+        case QEMU_FIRMWARE_FEATURE_INTEL_TDX:
         case QEMU_FIRMWARE_FEATURE_VERBOSE_DYNAMIC:
         case QEMU_FIRMWARE_FEATURE_VERBOSE_STATIC:
         case QEMU_FIRMWARE_FEATURE_NONE:
@@ -1532,6 +1558,7 @@ qemuFirmwareSanityCheck(const qemuFirmware *fw,
         case QEMU_FIRMWARE_FEATURE_AMD_SEV:
         case QEMU_FIRMWARE_FEATURE_AMD_SEV_ES:
         case QEMU_FIRMWARE_FEATURE_AMD_SEV_SNP:
+        case QEMU_FIRMWARE_FEATURE_INTEL_TDX:
         case QEMU_FIRMWARE_FEATURE_VERBOSE_DYNAMIC:
         case QEMU_FIRMWARE_FEATURE_VERBOSE_STATIC:
         case QEMU_FIRMWARE_FEATURE_LAST:
@@ -1660,7 +1687,7 @@ qemuFirmwareFillDomainLegacy(virQEMUDriver *driver,
         loader->format = VIR_STORAGE_FILE_RAW;
 
         /* Only use the default template path if one hasn't been
-         * provided by the user.
+         * provided by the user. Assume that the template is in 'raw' format.
          *
          * In addition to fully-custom templates, which are a valid
          * use case, we could simply be in a situation where
@@ -1681,8 +1708,13 @@ qemuFirmwareFillDomainLegacy(virQEMUDriver *driver,
          * In this case, the global default is to have Secure Boot
          * disabled, but the domain configuration explicitly enables
          * it, and we shouldn't overrule this choice */
-        if (!loader->nvramTemplate)
+        if (!loader->nvramTemplate) {
             loader->nvramTemplate = g_strdup(cfg->firmwares[i]->nvram);
+            loader->nvramTemplateFormat = VIR_STORAGE_FILE_RAW;
+        }
+
+        if (loader->nvramTemplateFormat == VIR_STORAGE_FILE_NONE)
+            loader->nvramTemplateFormat = VIR_STORAGE_FILE_RAW;
 
         VIR_DEBUG("decided on firmware '%s' template '%s'",
                   loader->path, NULLSTR(loader->nvramTemplate));
@@ -1967,6 +1999,7 @@ qemuFirmwareGetSupported(const char *machine,
             case QEMU_FIRMWARE_FEATURE_AMD_SEV:
             case QEMU_FIRMWARE_FEATURE_AMD_SEV_ES:
             case QEMU_FIRMWARE_FEATURE_AMD_SEV_SNP:
+            case QEMU_FIRMWARE_FEATURE_INTEL_TDX:
             case QEMU_FIRMWARE_FEATURE_ENROLLED_KEYS:
             case QEMU_FIRMWARE_FEATURE_SECURE_BOOT:
             case QEMU_FIRMWARE_FEATURE_VERBOSE_DYNAMIC:
